// NodeMap.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "NodeMap.h"
#include "ToString.h"
#include "gdiflush.h"
// CNodeMap

IMPLEMENT_DYNAMIC(CNodeMap, CStatic)


/****************************************************************************
*                             CNodeMap::CNodeMap
* Effect: 
*       Constructor/Destructor
****************************************************************************/

CNodeMap::CNodeMap()
{

}

CNodeMap::~CNodeMap()
{
}

/****************************************************************************
*                                   Colors
****************************************************************************/

#define NODE_COLOR    RGB(255, 255,   0)
#define PACKAGE_COLOR RGB(  0,   0, 255)
#define CPU_COLOR     RGB(255,   0, 255)


/****************************************************************************
*                                 Message Map
****************************************************************************/

BEGIN_MESSAGE_MAP(CNodeMap, CStatic)
    ON_WM_PAINT()
    ON_WM_SIZE()
    ON_WM_ERASEBKGND()
    ON_MESSAGE(WM_SETFONT, OnSetFont)
    ON_MESSAGE(WM_GETFONT, OnGetFont)
END_MESSAGE_MAP()

// CNodeMap message handlers

/****************************************************************************
*                         CNodeMap::BitMaskToNumbers
* Inputs:
*       UINT_PTR mask: Mask of processors
* Result: CString
*       The number(s) represented by the processor mask
*       For a single bit: e.g,   0x00000080  => "7"
*       For multiple bits, e.g., 0x00000003  => "0,1"
****************************************************************************/

CString CNodeMap::BitMaskToNumbers(UINT_PTR mask)
    {
     CString result;
     CString sep;
     
     for(UINT i = 0; i < sizeof(mask) * 8; i++)
        { /* add to mask */
         if( (mask & 1 << i) == 0)
             continue;
         result += sep;
         result += ToString(_T("%d"), i);
         sep = _T(",");
        } /* add to mask */
     return result;
    } // CNodeMap::BitMaskToNumbers

/****************************************************************************
*                             CNodeMap::DrawCores
* Inputs:
*       CDC & dc: DC to draw in
*       CRect & r: Place to draw them
*       std::vector<CoreInfo> Cores:  Core informatio
* Result: void
*       
* Effect: 
*       Draws the CPU cores
****************************************************************************/

void CNodeMap::DrawCores(CDC & dc, const CRect & r, const std::vector<CoreInfo> & Cores)
    {
     int InterCoreGap = 4 * ::GetSystemMetrics(SM_CXEDGE);
     int save = dc.SaveDC();
     int rows;
     int columns;

     int f1;
     int f2;
     switch(Cores.size())
        { /* cores */
#define COREASPECT(n, a, b) case n: f1 = a; f2 = b; break;
         COREASPECT(1,  1,  1);
         COREASPECT(2,  1,  2);
         COREASPECT(4,  2,  2);
         COREASPECT(6,  2,  3);
         COREASPECT(8,  4,  4);
         COREASPECT(10, 2,  5);
         COREASPECT(12, 3,  4);
         COREASPECT(14, 2,  7);
         COREASPECT(16, 4,  4);
         COREASPECT(18, 2,  9);
         COREASPECT(20, 4,  5);
         COREASPECT(22, 2, 11);
         COREASPECT(24, 4,  6);
         default:
            return;
#undef COREASPECT
        } /* cores */
     if(r.Height() > r.Width())
        { /* vertical */
         columns = f1;
         rows = f2;
        } /* vertical */
     else
        { /* horizontal */
         columns = f2;
         rows = f1;
        } /* horizontal */

     int HorizontalOverhead = (2 * InterCoreGap + (columns - 1) * InterCoreGap);
     int VerticalOverhead = (2 * InterCoreGap + (rows - 1) * InterCoreGap);

     int width = (r.Width() - HorizontalOverhead) / columns;
     int height = (r.Height() - VerticalOverhead) / rows;

     int size = min(width, height);

     CFont * f = GetFont();
     if(f == NULL)
        { /* no font */
         f = CFont::FromHandle((HFONT)::GetStockObject(DEFAULT_GUI_FONT));
        } /* no font */

     LOGFONT lf;
     f->GetLogFont(&lf);
     lf.lfHeight = (int)(0.8f * (double)size);
     CFont caption;
     caption.CreateFontIndirect(&lf);
     dc.SelectObject(&caption);

     CBrush br(CPU_COLOR);
     dc.SelectObject(&br);
     dc.SetBkMode(TRANSPARENT);

     int xgap = (r.Width() - (columns * size)) / (columns + 1);
     int ygap = (r.Height() - (rows * size)) / (rows + 1); 

     int i = 0;
     for(int x = 0; x < columns; x++)
        for(int y = 0; y < rows; y++)
           { /* output cores */
            CRect cpu(xgap + x * (size + xgap),
                      ygap + y * (size + ygap),
                      0, 0);
            cpu.left += r.left;
            cpu.top += r.top;
            cpu.right = cpu.left + size;
            cpu.bottom = cpu.top + size;
            dc.Rectangle(&cpu);
            GDI_FLUSH();

            CString num = BitMaskToNumbers(Cores[i].ProcessorMask);
            CRect text = cpu;
            dc.DrawText(num, &text, DT_CENTER | DT_VCENTER | DT_SINGLELINE);
            GDI_FLUSH();
            i++;
           } /* output cores */

     dc.RestoreDC(save);
    } // CNodeMap::DrawCores

/****************************************************************************
*                              CNodeMap::OnPaint
* Result: void
*       
* Effect: 
*       Draws the map
* Notes:
*
*   |<----------------client.Width()------------------->|
*   |                                                   |
*   +---------------------------------------------------+  -------
*   |                                                   |    ^  ^
*   |                     NodeWidth                     |    |  | TopMargin
*   |                     |<----->|                     |    |  v
*   |                     +-------+                     |    | ---
*   |                     |       |                     |    |  ^
*   |                     | node  |                     |    |  | NodeHeight
*   |                     |       |                     |    c  v
*   |                     +---+---+                     |    l ---
*   |                         |                         |    i  ^                     |
*   |              +----------+--------+                |    e  | GapMargin           |
*   |              |                   |                |    n  v                     v
*   |   +---------+---------+   +------+-----------+    |    t --------------------------
*   |   |   +----+          |   |    +----+        |    |    .  ^   ---------------------
*   |   |   |core|          |   |    |core|        |    |    H  |    |<--CoreSize.cy  ^
*   |   |   +----+       ---|   |    + - -+        |    |    e  |   ----              |
*   |   |                 ^ |   |    |core|        |    |    i  |                     |
*   |   |     InterCoreGap| |   |    +----+        |    |    g  | PackageSize.cy      CoreMargin
*   |   |                 V |   |    |<-->|        |    |    h  |
*   |   |   +----+       ---|   |      ||          |    |    t  |
*   |   |   |core|          |   | CoreSize.cy      |    |    |  |
*   |   |   +----+          |   |                  |    |    |  |
*   |   |package            |   |package           |    |    |  v
*   |   +-------------------+   +------------------+    |    | --- 
*   |   |                   |   |                       |    |  ^
*-->|   |<--PacakgeLeftMargin|PackageRightMargin-->|    |<-- |  | BottomMargin
*   |   |<--PackageSize.cx->|   |                       |    |  |
*   |                   --->|   |<---PagageGap          |    v  v
*   +---------------------------------------------------+ ---------
*                                             
*     node  package  shared   core
*  +------+---------+-------+-------+
*  |   0  |  00011  |   0   | 0001  |
*  +------+---------+-------+-------+
*  |   0  |  00011  |   0   | 0010  |
*  +------+---------+-------+-------+
*  |   1  |  01100  |   0   | 0100  |
*  +------+---------+-------+-------+
*  |   1  |  01100  |   0   | 1000  |
*  +------+---------+-------+-------+
*
****************************************************************************/

void CNodeMap::OnPaint()
    {
     CPaintDC dc(this);

     CRect client;
     int LeftMargin;
     int RightMargin;
     CSize PackageSize;
     CSize NodeSize;
     CSize CoreSize;
     int NodeGap = 0;
     int PackageGap = 0;
     int GapMargin;
     int NodeLeftMargin;
     int NodeRightMargin;
     int PackageLeftMargin;
     int PackageRightMargin;
     int TopMargin;
     int BottomMargin;

     GetClientRect(&client);

     dc.DrawEdge(&client, EDGE_BUMP, BF_ADJUST | BF_RECT);

     if(Nodes.empty())
        return;  // nothing to draw
     if(Packages.empty())
        return;  // nothing to draw

     int count = max(Packages.size(), Nodes.size());
     
     int gap = ::GetSystemMetrics(SM_CXVSCROLL); // randomly-chosen idea of nice spacing
     

     // |<--LeftMargin-->|<----NodeSize.cx--->|<--Gap-->|<----NodeSize.cx--->|...|<--RightMargin-->|
     // |<--LeftMargin-->|<--PackageSize.cx-->|<--Gap-->|<--PackageSize.cx-->|...|<--RightMargin-->|

     LeftMargin = gap;
     RightMargin = gap;
     int overhead = LeftMargin + RightMargin +
                    (count - 1) * gap; // inter-object gap

     int width = (client.Width() - overhead) / count;

     NodeSize.cx = width;
     NodeGap = (client.Width() - (Nodes.size() * width)) / (2 + (Nodes.size() - 1));
     NodeLeftMargin = NodeGap;
     NodeRightMargin = NodeGap;

     PackageSize.cx = width;
     PackageGap = (client.Width() - (Packages.size() * width)) / (2 + (Packages.size() - 1));
     PackageLeftMargin = PackageGap;
     PackageRightMargin = PackageGap;

     TopMargin = ::GetSystemMetrics(SM_CYHSCROLL);
     BottomMargin = TopMargin;
     GapMargin = TopMargin * 2;

     int height = (client.Height() - (TopMargin + BottomMargin + GapMargin)) / 2;
     
     PackageSize.cy = height;
     NodeSize.cy = height;

     CFont * f = GetFont();
     LOGFONT lf;
     f->GetLogFont(&lf);
     lf.lfHeight = (int) ((0.5) * (double)NodeSize.cy);
     CFont font;
     font.CreateFontIndirect(&lf);

     for(SIZE_T i = 0; i < Nodes.size(); i++)
        { /* show nodes */
         int save = dc.SaveDC();
         CBrush br(NODE_COLOR);
         dc.SelectObject(&br);

         dc.SetBkMode(TRANSPARENT);


         CRect r(NodeLeftMargin + (i * (NodeSize.cx + NodeGap)),
                 TopMargin, 
                 0, 0);
         r.right = r.left + NodeSize.cx;
         r.bottom = r.top + NodeSize.cy;
         Nodes[i].rect = r;

         
         CString caption = _T("Node");
         if(dc.GetTextExtent(caption).cx < r.Width())
            { /* write caption */
             int save2 = dc.SaveDC();
             dc.SelectStockObject(DEFAULT_GUI_FONT);
             dc.SetTextAlign(TA_LEFT | TA_BOTTOM);
             dc.TextOut(r.left, r.top - 2, caption); 
             GDI_FLUSH();
             dc.RestoreDC(save2);
            } /* write caption */

         dc.Rectangle(&r);
         GDI_FLUSH();
 
         dc.SelectObject(&font);
         CString s = ToString(_T("%d"), i);

         dc.DrawText(s, &r, DT_CENTER | DT_VCENTER | DT_SINGLELINE);

         GDI_FLUSH();
         dc.RestoreDC(save);
        } /* show nodes */

     for(SIZE_T i = 0; i < Packages.size(); i++)
        { /* show packages */
         int save = dc.SaveDC();
         CBrush br(PACKAGE_COLOR);
         dc.SelectObject(&br);

         CRect r(PackageLeftMargin + (i * (PackageSize.cx + PackageGap)),
                 TopMargin + NodeSize.cy + GapMargin,
                 PackageLeftMargin + (i * (PackageSize.cx + PackageGap)) + PackageSize.cx,
                 TopMargin + NodeSize.cy + GapMargin + PackageSize.cy);

         Packages[i].rect = r;

         CString caption = _T("Pkg");
         if(dc.GetTextExtent(caption).cx < r.Width() / 2)
            { /* draw caption */
             int save2 = dc.SaveDC();
             dc.SelectStockObject(DEFAULT_GUI_FONT);
             dc.SetTextAlign(TA_LEFT | TA_BOTTOM);
             dc.TextOut(r.left, r.top - 2, caption); 
             GDI_FLUSH();
             dc.RestoreDC(save2);
            } /* draw caption */

         dc.Rectangle(&r);
         GDI_FLUSH();
         DrawCores(dc, r, Packages[i].Cores);
         dc.RestoreDC(save);
        } /* show packages */

     // Draw lines from packages to nodes
     for(SIZE_T pkg = 0; pkg < Packages.size(); pkg++)
        for(SIZE_T node = 0; node < Nodes.size(); node++)
           { /* draw connection */
            if(Nodes[node].ProcessorMask == Packages[pkg].ProcessorMask)
               { /* connect node to processor */
                dc.MoveTo(Nodes[node].rect.left + Nodes[node].rect.Width() / 2,
                          Nodes[node].rect.bottom);
                dc.LineTo(Packages[pkg].rect.left + Packages[pkg].rect.Width() / 2,
                          Packages[pkg].rect.top);
               } /* connect node to processor */
            GDI_FLUSH();
           } /* draw connection */
   } // CNodeMap::OnPaint


/****************************************************************************
*                              CNodeMap::SetInfo
* Inputs:
*       std::vector<SYSTEM_LOGICAL_PROCESSOR_INFORMATION> & LPI:
* Result: void
*       
* Effect: 
*       Creates the internal display structure
****************************************************************************/

void CNodeMap::SetInfo(std::vector<SYSTEM_LOGICAL_PROCESSOR_INFORMATION> & LPI)
    {
     Nodes.clear();
     Packages.clear();

     //****************************************************************
     // Pass 1: Create nodes and packages
     //****************************************************************

     for(SIZE_T i = 0; i < LPI.size(); i++)
        { /* scan elements pass 1*/
         switch(LPI[i].Relationship)
            { /* relationship */
             case RelationCache:
                // ignored
                break;
             case RelationNumaNode:
                Nodes.push_back(NodeInfo(LPI[i].NumaNode.NodeNumber, LPI[i].ProcessorMask));
                break;
             case RelationProcessorCore:
                // Wait for pass 2
                break;
             case RelationProcessorPackage:
                { /* add package */
                 Packages.push_back(PackageInfo(LPI[i].ProcessorMask));
                } /* add package */
                break;
            } /* relationship */
        } /* scan elements pass 1*/

     //****************************************************************
     // Pass 2: Processor core maps
     //****************************************************************
     
     for(SIZE_T i = 0; i < LPI.size(); i++)
        { /* scan elements pass 2*/
         switch(LPI[i].Relationship)
            { /* relationship */
             case RelationCache:
                // ignored
                break;
             case RelationNumaNode:
                // Handled in pass 1
                break;
             case RelationProcessorCore:
                { /* core info */
                 for(SIZE_T pk = 0; pk < Packages.size(); pk++)
                    { /* scan packages */
                     if( (Packages[pk].ProcessorMask & LPI[i].ProcessorMask) != 0)
                        { /* processor in package */
                         Packages[pk].Cores.push_back(CoreInfo(LPI[i].ProcessorMask, LPI[i].ProcessorCore.Flags));
                        } /* processor in package */
                    } /* scan packages */
                } /* core info */
                break;
             case RelationProcessorPackage:
                // Handled in pass 1
                break;
            } /* relationship */
         
        } /* scan elements pass 2*/
    } // CNodeMap::SetInfo

/****************************************************************************
*                              CNodeMap::OnSize
* Inputs:
*       UINT nType:
*       int cx:
*       int cy:
* Result: void
*       
* Effect: 
*       Invalidates the area
****************************************************************************/

void CNodeMap::OnSize(UINT nType, int cx, int cy)
    {
     CStatic::OnSize(nType, cx, cy);
     Invalidate();
    } // CNodeMap::OnSize

/****************************************************************************
*                           CNodeMap::OnEraseBkgnd
* Inputs:
*       CDC * pDC:
* Result: BOOL
*       TRUE, always
* Effect: 
*       Erases the background
****************************************************************************/

BOOL CNodeMap::OnEraseBkgnd(CDC * pDC)
    {
     CRect client;
     GetClientRect(&client);
     pDC->FillSolidRect(&client, ::GetSysColor(COLOR_3DFACE));
     return TRUE;
    } // CNodeMap::OnEraseBkgnd

/****************************************************************************
*                             CNodeMap::OnSetFont
* Inputs:
*       WPARAM: (WPARAM)(HFONT)
*       LPARAM: MAKELPARAM(BOOL, ?): TRUE to immediately redraw
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Sets the font variable
****************************************************************************/

LRESULT CNodeMap::OnSetFont(WPARAM wParam, LPARAM lParam)
    {
     Font = (HFONT)wParam;
     if(LOWORD(lParam))
        { /* redraw */
         Invalidate();
         UpdateWindow();
        } /* redraw */
     return 0;
    } // CNodeMap::OnSetFont

/****************************************************************************
*                             CNodeMap::OnGetFont
* Inputs:
*       WPARAM: ignored
*       LPARAM: ignored
* Result: (LRESULT)(HFONT)
*       The current font
****************************************************************************/

LRESULT CNodeMap::OnGetFont(WPARAM, LPARAM)
    {
     return (LRESULT)Font;
    } // CNodeMap::OnGetFont
