// NumaExplorerDlg.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "NumaExplorerDlg.h"
#include "ToString.h"
#include "TraceEvent.h"
#include "NumaTags.h"
#include "..\XMLBuilder\XML.h"
#include "..\XMLBuilder\XMLBuilder.h"
#include "ErrorString.h"

#pragma comment(lib, "psapi.lib")

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


/****************************************************************************
*                                  About Box
****************************************************************************/

// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
   {
    public:
        CAboutDlg();

        // Dialog Data
        enum { IDD = IDD_ABOUTBOX };

        protected:
        virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

        // Implementation
        protected:
           DECLARE_MESSAGE_MAP()
   };

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
   {
   }

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
   {
    CDialog::DoDataExchange(pDX);
   }

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
END_MESSAGE_MAP()

/****************************************************************************
*                              class BufferInfo
****************************************************************************/

class BufferInfo {
    public:
       BufferInfo() { Buffer = NULL; }
       virtual ~BufferInfo() {
          if(Buffer != NULL)
             ::VirtualFree(Buffer, 0, MEM_RELEASE);
       }
       PVOID Buffer;
};


/****************************************************************************
*                     CNumaExplorerDlg::CNumaExplorerDlg
* Inputs:
*       CWnd * pParent: Parent window
*       
* Effect: 
*       Constructor
****************************************************************************/

CNumaExplorerDlg::CNumaExplorerDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CNumaExplorerDlg::IDD, pParent)
   {
    m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
    NumaHighestNodeNumber = 0;
    SYSTEM_INFO SystemInfo;
    GetSystemInfo (&SystemInfo);
    PageSize = SystemInfo.dwPageSize;

    Data = NULL;
   }


/****************************************************************************
*                       CNumaExplorerDlg::DoDataExchange
* Inputs:
*       CDataExchange * pDX:
* Result: void
*       
* Effect: 
*       Binds controls to variables
****************************************************************************/

void CNumaExplorerDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_NUMA_HIGHEST_NODE, c_NumaHighestNodeNumber);
    DDX_Control(pDX, IDC_SYSTEM_INFO, c_SYSTEM_INFO);
    DDX_Control(pDX, IDC_NUMA, c_Numa);
    DDX_Control(pDX, IDC_NUMAMASK, c_NumaNodeProcessorMask);
    DDX_Control(pDX, IDC_NUMAMEMORY, c_NumaAvailableMemoryNode);
    DDX_Control(pDX, IDC_GLPI, c_GLPI);
    DDX_Control(pDX, IDC_FRAME, c_Frame);
    DDX_Control(pDX, IDC_NUMA_MEMORY_MAP, c_NumaMemoryMap);
    DDX_Control(pDX, IDC_PAGES, c_Pages);
    DDX_Control(pDX, IDC_LOG, c_Log);
    DDX_Control(pDX, IDC_NODEINFO, c_NodeInfo);
    DDX_Control(pDX, IDC_NODE_MAP, c_NodeMap);
    DDX_Control(pDX, IDC_XML, c_XML);
}

/****************************************************************************
*                                 Message Map
****************************************************************************/

BEGIN_MESSAGE_MAP(CNumaExplorerDlg, CDialog)
        ON_WM_SYSCOMMAND()
//      ON_WM_PAINT()
        ON_WM_QUERYDRAGICON()
        //}}AFX_MSG_MAP
        ON_WM_CLOSE()
        ON_WM_SIZE()
        ON_BN_CLICKED(IDC_REFRESH, &CNumaExplorerDlg::OnBnClickedRefresh)
        ON_WM_GETMINMAXINFO()
        ON_COMMAND(ID_HELP_ABOUTNUMAEXPLORER, &CNumaExplorerDlg::OnHelpAboutNumaExplorer)
        ON_COMMAND(ID_FILE_EXIT, &CNumaExplorerDlg::OnFileExit)
        ON_COMMAND(ID_EDIT_CLEARLOG, &CNumaExplorerDlg::OnEditClearLog)
        ON_COMMAND(ID_FILE_SAVEXMLFILE, &CNumaExplorerDlg::OnFileSaveXMLFile)
        ON_COMMAND(ID_FILE_LOADXMLFILE, &CNumaExplorerDlg::OnFileLoadXMLFile)
        ON_WM_DESTROY()
        ON_REGISTERED_MESSAGE(CXMLDisplay::UWM_QUERY_NODE, &CNumaExplorerDlg::OnQueryNode)
        ON_WM_INITMENUPOPUP()
END_MESSAGE_MAP()

/****************************************************************************
*                        CNumaExplorerDlg::DumpRegion
* Inputs:
*       UCHAR Cpu: CPU this allocation is associated with
*       UCHAR Nodes: Number of nodes in system
*       PVOID StartPtr: Start of region
*       PVOID EndPtr: End of region
*       DWORD Node: Node it is associated with
*       BOOL Valid: Validity
* Result: void
*       
* Effect: 
*       Logs the region data
****************************************************************************/

void CNumaExplorerDlg::DumpRegion(UCHAR Cpu, UCHAR Nodes, PVOID StartPtr, PVOID EndPtr, DWORD Node, BOOL Valid, XMLTag * VAEN)
    {
     c_NumaMemoryMap.AddRange(Cpu, Nodes, (UINT_PTR)StartPtr, (UINT_PTR)(EndPtr), (UCHAR)Node, Valid);

     XMLTag * alloc = new XMLTag(NUMATags::Tag::NUMA, NUMATags::DataTags::VirtualAllocExNumaTags::AllocationTags::Allocation);
     VAEN->AddChild(alloc);

     CString s;

     s.Format(_T("%d"), Cpu);
     alloc->SetValue(NUMATags::DataTags::VirtualAllocExNumaTags::AllocationTags::Attributes::CPU, s);
     
     s.Format(_T("0x%p"), StartPtr);
     alloc->SetValue(NUMATags::DataTags::VirtualAllocExNumaTags::AllocationTags::Attributes::Low, s);

     s.Format(_T("0x%p"), EndPtr);
     alloc->SetValue(NUMATags::DataTags::VirtualAllocExNumaTags::AllocationTags::Attributes::High, s);

     s.Format(_T("%d"), Node);
     alloc->SetValue(NUMATags::DataTags::VirtualAllocExNumaTags::AllocationTags::Attributes::Node, s);

     alloc->SetValue(NUMATags::DataTags::VirtualAllocExNumaTags::AllocationTags::Attributes::Valid, Valid ? _T("true") : _T("false"));

     DWORD_PTR StartPage = ((DWORD_PTR)StartPtr)/PageSize;
     DWORD_PTR EndPage   = ((DWORD_PTR)EndPtr)/PageSize;
     DWORD_PTR NumPages  = (EndPage - StartPage) + 1;
     
     if(Valid)
        s.Format(_T("CPU %d: %p - %p (%6Iu pages): node %u"), Cpu, StartPtr, EndPtr, NumPages, Node);
     else
        s.Format(_T("CPU %d: %p - %p (%6Iu pages): no valid pages"), Cpu, StartPtr, EndPtr, NumPages);
     c_NodeInfo.AddString(s);
    } // CNumaExplorerDlg::DumpRegion

/****************************************************************************
*                     CNumaExplorerDlg::DumpNumaNodeInfo
* Inputs:
*       UCHAR Cpu: CPU #
*       UCHAR Nodes: Number of nodes in system
*       PVOID Buffer: Pointer to buffer
*       SIZE_T Size:  Length of buffer
*       XMLTag * VAEN: Node to add it to
* Result: void
*       
* Effect: 
*       Creates the region map
****************************************************************************/

void CNumaExplorerDlg::DumpNumaNodeInfo(UCHAR Cpu, UCHAR Nodes, PVOID Buffer, SIZE_T Size, XMLTag * VAEN)
    {
     DWORD_PTR StartPage = ((DWORD_PTR)Buffer)/PageSize;
     DWORD_PTR EndPage   = ((DWORD_PTR)Buffer + Size - 1)/PageSize;
     DWORD_PTR NumPages  = (EndPage - StartPage) + 1;

     PCHAR StartPtr = (PCHAR)(PageSize*StartPage);

     std::vector<PSAPI_WORKING_SET_EX_INFORMATION> WsInfo;
     WsInfo.resize(NumPages);

     for (DWORD_PTR i = 0; i < NumPages; i++)
        {
         WsInfo[i].VirtualAddress = StartPtr + (i * PageSize);
        }
     
     BOOL bResult = ::QueryWorkingSetEx(
                                        GetCurrentProcess(),
                                        &WsInfo[0],
                                        NumPages * sizeof(PSAPI_WORKING_SET_EX_INFORMATION)
                                       );
     if (!bResult)
        {
         DWORD err = ::GetLastError();
         c_Log.AddString(new TraceError(TraceEvent::None, IDS_QUERYWORKINGSETEX_FAILED));
         c_Log.AddString(new TraceFormatMessage(TraceEvent::None, err));
         return;
        }

     PCHAR RegionStart;
     BOOL  RegionIsValid;
     DWORD RegionNode;

     for (DWORD_PTR i = 0; i < NumPages; i++)
        { /* Show runs */
         PCHAR Address = (PCHAR)WsInfo[i].VirtualAddress;
         BOOL  IsValid = WsInfo[i].VirtualAttributes.Valid;
         DWORD Node    = WsInfo[i].VirtualAttributes.Node;

         if (i == 0)
            { /* first run */
             RegionStart   = Address;
             RegionIsValid = IsValid;
             RegionNode    = Node;
            } /* first run */

         if (IsValid != RegionIsValid || Node != RegionNode)
            { /* run change */
             DumpRegion(Cpu, Nodes, RegionStart, Address - 1, RegionNode, IsValid, VAEN);

             RegionStart   = Address;
             RegionIsValid = IsValid;
             RegionNode    = Node;
            } /* run change */

         if (i == (NumPages - 1))
            { /* end of runs */
             DumpRegion(Cpu, Nodes, RegionStart, Address + PageSize - 1, Node, IsValid, VAEN);
            } /* end of runs */
        } /* Show runs */

    } // CNumaExplorerDlg::DumpNumaNodeInfo

/****************************************************************************
*                         CNumaExplorerDlg::NumaAlloc
* Inputs:
*       UINT Pages: The number of pages to allocate
*       XMLTag * VAEN: <NUMA:VirtualAllocExNuma/>
* Result: void
*       
* Effect: 
*       Does a sample NUMA allocation
****************************************************************************/

void CNumaExplorerDlg::NumaAlloc(UINT Pages, XMLTag * VAEN)
    {
     ULONG HighestNodeNumber;
     SIZE_T AllocationSize = PageSize * Pages;

     if (!::GetNumaHighestNodeNumber (&HighestNodeNumber))
        { /* GetNumaHighestNodeNumber failed */
         DWORD err = GetLastError();
         c_Log.AddString(new TraceFormatError(TraceEvent::None, IDS_GETNUMAHIGHESTNODE_FAILED));
         c_Log.AddString(new TraceFormatMessage(TraceEvent::None, err));
         return;
        } /* GetNumaHighestNodeNumber failed */

     if(HighestNodeNumber == 0)
        { /* not NUMA */
         c_Log.AddString(new TraceFormatError(TraceEvent::None, IDS_NOT_NUMA));
         return;
        } /* not NUMA */

     //****************************************************************
     // Allocate array of pointers to memory blocks.
     //****************************************************************

     std::vector<BufferInfo> Buffers;
     Buffers.resize(NumberOfProcessors);

     //****************************************************************
     // For each processor, get its associated NUMA node and allocate some memory from it.
     //****************************************************************

     for (UCHAR i = 0; i < NumberOfProcessors; i++)
        { /* Per-processor allocation */
         UCHAR NodeNumber;

         if (! ::GetNumaProcessorNode(i, &NodeNumber))
            {
             DWORD err = ::GetLastError();
             c_Log.AddString(new TraceError(TraceEvent::None, IDS_GETNUMAPROCESSORNODE_FAILED));
             c_Log.AddString(new TraceFormatMessage(TraceEvent::None, err));
             return;
            }

         //_tprintf (_T("CPU %u: node %u\n"), (ULONG)i, NodeNumber);

         PCHAR Buffer = (PCHAR)::VirtualAllocExNuma(
            GetCurrentProcess(),
            NULL,
            AllocationSize,
            MEM_RESERVE | MEM_COMMIT,
            PAGE_READWRITE,
            NodeNumber
            );

         if (Buffer == NULL)
            {
             DWORD err = ::GetLastError();
             if(sizeof(UINT_PTR) == sizeof(DWORD))
                c_Log.AddString(new TraceFormatError(TraceEvent::None, IDS_VIRTUALALLOCEXNUMA_FAILED32, NodeNumber, AllocationSize));
             else
                c_Log.AddString(new TraceFormatError(TraceEvent::None, IDS_VIRTUALALLOCEXNUMA_FAILED64, NodeNumber, AllocationSize));
             c_Log.AddString(new TraceFormatMessage(TraceEvent::None, err));
             return;
            }

         PCHAR BufferEnd = Buffer + AllocationSize - 1;
         SIZE_T NumPages = ((SIZE_T)BufferEnd)/PageSize - ((SIZE_T)Buffer)/PageSize + 1;

         c_Log.AddString(new TraceFormatComment(TraceEvent::None, IDS_ALLOCATED_VM, i, Buffer, BufferEnd, NumPages, NodeNumber));

         Buffers[i].Buffer = Buffer;

         //****************************************************************
         // At this point, virtual pages are allocated but no valid physical
         // pages are associated with them yet.
         //
         // The FillMemory call below will touch every page in the buffer, faulting
         // them into our working set. When this happens physical pages will be allocated
         // from the preferred node we specified in VirtualAllocExNuma, or any node
         // if the preferred one is out of pages.
         //****************************************************************

         ::FillMemory (Buffers[i].Buffer, AllocationSize, 'x');

         //****************************************************************
         // Check the actual node number for the physical pages that are still valid
         // (if system is low on physical memory, some pages could have been trimmed already).
         //****************************************************************

         DumpNumaNodeInfo (i, (UCHAR)(HighestNodeNumber + 1), Buffers[i].Buffer, AllocationSize, VAEN);
        } /* Per-processor allocation */

    } // CNumaExplorerDlg::NumaAlloc

/****************************************************************************
*                          CNumaExplorerDlg::Refresh
* Result: void
*       
* Effect: 
*       Updates the data
****************************************************************************/

void CNumaExplorerDlg::Refresh()
    {
     delete Data;
     Data = NULL;

     Data = new XMLTag(NUMATags::Tag::NUMA, NUMATags::DataTags::Data);
     c_Log.AddString(new TraceComment(TraceEvent::None, IDS_REFRESH));
     
     if(!GetNumaHighestNodeNumber(&NumaHighestNodeNumber))
        { /* node number failed */
         DWORD err = ::GetLastError();
         c_Log.AddString(new TraceError(TraceEvent::None, IDS_GETNUMAHIGHESTNODENUMBER_FAILED));
         c_Log.AddString(new TraceFormatMessage(TraceEvent::None, err));
         CString s(MAKEINTRESOURCE(IDS_FALSE));
         c_NumaHighestNodeNumber.SetWindowText(s);
        } /* node number failed */
     else
        { /* node number succeeded */
         CString s = ToString(_T("%u"), NumaHighestNodeNumber);
         c_NumaHighestNodeNumber.SetWindowText(s);
         XMLTag * GNHNN = new XMLTag(NUMATags::Tag::NUMA, NUMATags::DataTags::GetNumaHighestNodeNumberTags::GetNumaHighestNodeNumber);
         Data->AddChild(GNHNN);
         GNHNN->SetValue(NUMATags::DataTags::GetNumaHighestNodeNumberTags::Attributes::Value, s);
        } /* node number succeeded */

     { /* Get System Info */
      SYSTEM_INFO SystemInfo;
      GetSystemInfo (&SystemInfo);
      NumberOfProcessors = SystemInfo.dwNumberOfProcessors;

      XMLTag * sysinfo = new XMLTag(NUMATags::Tag::NUMA, NUMATags::DataTags::SYSTEM_INFOTags::SYSTEM_INFO);
      Data->AddChild(sysinfo);

#define ADDSYSINFO(fmt, x) {                                                      \
      CString s = ToString(fmt, SystemInfo.x);                                    \
      sysinfo->SetValue(NUMATags::DataTags::SYSTEM_INFOTags::Attributes::x, s);   \
      CString t;                                                                  \
      t.Format(_T(#x) _T(": ") fmt, SystemInfo.x);                                \
      int n = c_SYSTEM_INFO.AddString(t);                                         \
      c_SYSTEM_INFO.SetItemData(n, offsetof(SYSTEM_INFO, x));                     \
      }

      ADDSYSINFO(_T("%u"), wProcessorArchitecture);
      CString s;
      switch(SystemInfo.wProcessorArchitecture)
         { /* wProcessorArchitecture */
#define CASEARCH(x) case x: c_SYSTEM_INFO.AddString(_T("wProcessorArchitecture: ") _T(#x));  break
          CASEARCH(PROCESSOR_ARCHITECTURE_AMD64);
          CASEARCH(PROCESSOR_ARCHITECTURE_IA32_ON_WIN64);
          CASEARCH(PROCESSOR_ARCHITECTURE_IA64);
          CASEARCH(PROCESSOR_ARCHITECTURE_INTEL);
          CASEARCH(PROCESSOR_ARCHITECTURE_UNKNOWN);
#undef CASEARCH
         } /* wProcessorArchitecture */

      ADDSYSINFO(_T("%u"), dwPageSize);
      ADDSYSINFO(_T("0x%p"), lpMinimumApplicationAddress);
      ADDSYSINFO(_T("0x%p"), lpMaximumApplicationAddress);
      if(sizeof(UINT_PTR) == sizeof(UINT))
         { /* 32-bit */
          ADDSYSINFO(_T("%u"), dwActiveProcessorMask);
         } /* 32-bit */
      else
         { /* 64-bit */
          ADDSYSINFO(_T("%I64u"), dwActiveProcessorMask);
         } /* 64-bit */
      ADDSYSINFO(_T("%u"), dwNumberOfProcessors);
      ADDSYSINFO(_T("%u"), dwProcessorType);
      switch(SystemInfo.dwProcessorType)
         { /* dwProcessorType */
#define CASEPROCESSOR(x) case x: c_SYSTEM_INFO.AddString(_T("dwProcessorType: ") _T(#x));  break
          CASEPROCESSOR(PROCESSOR_INTEL_386);
          CASEPROCESSOR(PROCESSOR_INTEL_486);
          CASEPROCESSOR(PROCESSOR_INTEL_PENTIUM);
          CASEPROCESSOR(PROCESSOR_INTEL_IA64);
          CASEPROCESSOR(PROCESSOR_AMD_X8664);
#undef CASEPROCESSOR    
         } /* dwProcessorType */
      ADDSYSINFO(_T("%u"), dwAllocationGranularity);
      ADDSYSINFO(_T("%u"), wProcessorLevel);
      ADDSYSINFO(_T("%u"), wProcessorRevision);
      
#undef ADDSYSINFO
      for(int i = 0; i < c_SYSTEM_INFO.GetCount(); i++)
         { /* select? */
          size_t offset = c_SYSTEM_INFO.GetItemData(i);
          if(offset == offsetof(SYSTEM_INFO, dwNumberOfProcessors))
             { /* found it */
              c_SYSTEM_INFO.SetCurSel(i);
              break;
             } /* found it */
         } /* select? */
      if(c_SYSTEM_INFO.GetCurSel() < 0)
         c_SYSTEM_INFO.SetCurSel(0);
     } /* Get System Info */

     c_Numa.ResetContent();

     XMLTag * GNPN = new XMLTag(NUMATags::Tag::NUMA, NUMATags::DataTags::GetNumaProcessorNodeTags::GetNumaProcessorNode);
     Data->AddChild(GNPN);

     for(UCHAR i = 0; i < NumberOfProcessors; i++)
        { /* scan nodes */
         UCHAR NodeNumber;
         ::GetNumaProcessorNode(i, &NodeNumber);
         XMLTag * PN = new XMLTag(NUMATags::Tag::NUMA, NUMATags::DataTags::GetNumaProcessorNodeTags::ProcessorNodeTags::ProcessorNode);
         GNPN->AddChild(PN);
         CString s;
         s.Format(_T("%d"), i);
         PN->SetValue(NUMATags::DataTags::GetNumaProcessorNodeTags::ProcessorNodeTags::Attributes::Processor, s);
         s.Format(_T("%d"), NodeNumber);
         PN->SetValue(NUMATags::DataTags::GetNumaProcessorNodeTags::ProcessorNodeTags::Attributes::Node, s);
         s.Format(_T("pr %2d: node %d"), i, NodeNumber);
         c_Numa.AddString(s);
        } /* scan nodes */

     c_NumaNodeProcessorMask.ResetContent();

     XMLTag * GNNPM = new XMLTag(NUMATags::Tag::NUMA, NUMATags::DataTags::GetNumaProcessorMaskTags::GetNumaProcessorMask);
     Data->AddChild(GNNPM);

     for(UCHAR i = 0; i <= NumaHighestNodeNumber; i++)
        { /* scan nodes */
         ULONGLONG mask;
         ::GetNumaNodeProcessorMask(i, &mask);

         XMLTag * PM = new XMLTag(NUMATags::Tag::NUMA, NUMATags::DataTags::GetNumaProcessorMaskTags::ProcessorMaskTags::ProcessorMask);
         GNNPM->AddChild(PM);
         
         CString s;
         s.Format(_T("0x%I64x"), mask);
         PM->SetValue(NUMATags::DataTags::GetNumaProcessorMaskTags::ProcessorMaskTags::Attributes::ProcessorMask, s);
         s.Format(_T("%d"), i);
         PM->SetValue(NUMATags::DataTags::GetNumaProcessorMaskTags::ProcessorMaskTags::Attributes::Node, s);

         s.Format(_T("node %2d: mask 0x%016I64X"), i, mask);
         c_NumaNodeProcessorMask.AddString(s);
        } /* scan nodes */

     XMLTag * GNAMN = new XMLTag(NUMATags::Tag::NUMA, NUMATags::DataTags::GetNumaAvailableMemoryNodeTags::GetNumaAvailableMemoryNode);
     Data->AddChild(GNAMN);
     

     c_NumaAvailableMemoryNode.ResetContent();
     for(UCHAR i = 0; i <= NumaHighestNodeNumber; i++)
        { /* scan nodes */
         ULONGLONG bytes;
         if(!::GetNumaAvailableMemoryNode(i, &bytes))
            { /* failed */
             DWORD err = ::GetLastError();
             CString s;
             s.Format(_T("node %2d: FALSE (%d)"), i, err);
             c_Log.AddString(new TraceError(TraceEvent::None, IDS_GETNUMAAVAILABLEMEMORYNODE_FAILED));
             c_Log.AddString(new TraceFormatMessage(TraceEvent::None, err));
             c_NumaAvailableMemoryNode.AddString(s);
            } /* failed */
         else
            { /* succeeded */
             CString s;
             bytes /= 1024;

             XMLTag * AM = new XMLTag(NUMATags::Tag::NUMA, NUMATags::DataTags::GetNumaAvailableMemoryNodeTags::NumaAvailableMemoryTags::NumaAvailableMemory);
             GNAMN->AddChild(AM);
             s.Format(_T("%d"), i);
             AM->SetValue(NUMATags::DataTags::GetNumaAvailableMemoryNodeTags::NumaAvailableMemoryTags::Attributes::Node, s);
             s.Format(_T("%I64u"), bytes);
             AM->SetValue(NUMATags::DataTags::GetNumaAvailableMemoryNodeTags::NumaAvailableMemoryTags::Attributes::KBytes, s);

             s.Format(_T("node %2d: mem %I64u K"), i, bytes);
             c_NumaAvailableMemoryNode.AddString(s);
            } /* succeeded */
        } /* scan nodes */

     std::vector<SYSTEM_LOGICAL_PROCESSOR_INFORMATION> LPI;
     DWORD len = 0;
     
     XMLTag * GLPI = new XMLTag(NUMATags::Tag::NUMA, NUMATags::DataTags::GetLogicalProcessorInformationTags::GetLogicalProcessorInformation);
     Data->AddChild(GLPI);

     c_GLPI.ResetContent();
     if(!::GetLogicalProcessorInformation(NULL, &len))
        { /* has logical processor info */
         LPI.resize(len / sizeof(SYSTEM_LOGICAL_PROCESSOR_INFORMATION));
         ::GetLogicalProcessorInformation(&LPI[0], &len);
         //*****************************************************************
         // ProcessorMask      | Relationship
         //--------------------+--------------------------|-----------------
         // 0000000000000000   | RelationCache            | CACHE_DESCRIPTOR
         //                    |                          |    BYTE Level
         //                    |                          |    BYTE Associativity
         //                    |                          |    WORD LineSize
         //                    |                          |    DWORD Size
         //                    |                          |    PROCESSOR_CACHE_TYPE CacheUnified | CacheInstruction | CacheData | CacheTrace
         //                    | RelationNumaNode         | DWORD NodeNumber
         //                    | RelationProcessorCore    | BYTE Flags
         //                    | RelationProcessorPackage | ----

         for(SIZE_T i = 0; i < LPI.size(); i++)
            { /* add one */
             PSYSTEM_LOGICAL_PROCESSOR_INFORMATION item = new SYSTEM_LOGICAL_PROCESSOR_INFORMATION;
             memcpy_s(item, sizeof(SYSTEM_LOGICAL_PROCESSOR_INFORMATION), &LPI[i], sizeof(SYSTEM_LOGICAL_PROCESSOR_INFORMATION));

             switch(item->Relationship)
                { /* relationship */
                 case RelationNumaNode:
                    { /* RelationNumaNode */
                     XMLTag * RNN = new XMLTag(NUMATags::Tag::NUMA, NUMATags::DataTags::GetLogicalProcessorInformationTags::RelationNumaNodeTags::RelationNumaNode);
                     GLPI->AddChild(RNN);
                     CString s;
                     s.Format(_T("0x%p"), item->ProcessorMask);
                     RNN->SetValue(NUMATags::DataTags::GetLogicalProcessorInformationTags::RelationNumaNodeTags::Attributes::ProcessorMask, s);
                     s.Format(_T("%d"), item->NumaNode.NodeNumber);
                     RNN->SetValue(NUMATags::DataTags::GetLogicalProcessorInformationTags::RelationNumaNodeTags::Attributes::Node, s);
                    } /* RelationNumaNode */
                    break;
                 case RelationCache:
                    { /* RelationCache */
                     XMLTag * RC = new XMLTag(NUMATags::Tag::NUMA, NUMATags::DataTags::GetLogicalProcessorInformationTags::RelationCacheTags::RelationCache);
                     GLPI->AddChild(RC);
                     CString s;

                     s.Format(_T("0x%p"), item->ProcessorMask);
                     RC->SetValue(NUMATags::DataTags::GetLogicalProcessorInformationTags::RelationCacheTags::Attributes::ProcessorMask, s);

                     s.Format(_T("%d"), item->Cache.Level);
                     RC->SetValue(NUMATags::DataTags::GetLogicalProcessorInformationTags::RelationCacheTags::Attributes::Level, s);
                     
                     s.Format(_T("%d"), item->Cache.Associativity);
                     RC->SetValue(NUMATags::DataTags::GetLogicalProcessorInformationTags::RelationCacheTags::Attributes::Associativity, s);

                     s.Format(_T("%d"), item->Cache.LineSize);
                     RC->SetValue(NUMATags::DataTags::GetLogicalProcessorInformationTags::RelationCacheTags::Attributes::LineSize, s);

                     s.Format(_T("%d"), item->Cache.Type);
                     RC->SetValue(NUMATags::DataTags::GetLogicalProcessorInformationTags::RelationCacheTags::Attributes::Type, s);

                     switch(item->Cache.Type)
                        { /* type */
#define CASE_TYPE(x) case x: s = _T(#x); break
                         CASE_TYPE(CacheUnified);
                         CASE_TYPE(CacheInstruction);
                         CASE_TYPE(CacheData);
                         CASE_TYPE(CacheTrace);
                         default:
                            s = ToString(_T("%d"), item->Cache.Type);
                            break;
#undef CASE_TYPE
                        } /* type */
                     RC->SetValue(NUMATags::DataTags::GetLogicalProcessorInformationTags::RelationCacheTags::Attributes::TypeName, s);
                    } /* RelationCache */
                    break;
                 case RelationProcessorCore:
                    { /* RelationProcessorCore */
                     XMLTag * RPC = new XMLTag(NUMATags::Tag::NUMA, NUMATags::DataTags::GetLogicalProcessorInformationTags::RelationProcessorCoreTags::RelationProcessorCore);
                     GLPI->AddChild(RPC);
                     CString s;
                     s.Format(_T("0x%p"), item->ProcessorMask);
                     RPC->SetValue(NUMATags::DataTags::GetLogicalProcessorInformationTags::RelationProcessorCoreTags::Attributes::ProcessorMask, s);
                     s.Format(_T("0x%02x"), item->ProcessorCore.Flags);
                     RPC->SetValue(NUMATags::DataTags::GetLogicalProcessorInformationTags::RelationProcessorCoreTags::Attributes::Flags, s);
                    } /* RelationProcessorCore */
                    break;
                 case RelationProcessorPackage:
                    { /* RelationProcessorPackage */
                     XMLTag * RPP = new XMLTag(NUMATags::Tag::NUMA, NUMATags::DataTags::GetLogicalProcessorInformationTags::RelationProcessorPackageTags::RelationProcessorPackage);
                     GLPI->AddChild(RPP);
                     CString s;
                     s.Format(_T("0x%p"), item->ProcessorMask);
                     RPP->SetValue(NUMATags::DataTags::GetLogicalProcessorInformationTags::RelationProcessorCoreTags::Attributes::ProcessorMask, s);
                    } /* RelationProcessorPackage */
                    break;
                } /* relationship */

             c_GLPI.AddString((LPCTSTR)item);
            } /* add one */
         c_NodeMap.SetInfo(LPI);
        } /* has logical processor info */
     
     
     { /* allocate NUMA pages */
      c_NumaMemoryMap.Clear();
      c_NodeInfo.ResetContent();
      CString s;
      c_Pages.GetWindowText(s);
      UINT pages = _ttoi(s);

      XMLTag * VAEN = new XMLTag(NUMATags::Tag::NUMA, NUMATags::DataTags::VirtualAllocExNumaTags::VirtualAllocExNuma);
      Data->AddChild(VAEN);
      NumaAlloc(pages, VAEN);
     } /* allocate NUMA pages */

    } // CNumaExplorerDlg::Refresh

/****************************************************************************
*                        CNumaExplorerDlg::OnInitDialog
* Result: BOOL
*       
* Effect: 
*       Initializes
****************************************************************************/

BOOL CNumaExplorerDlg::OnInitDialog()
   {
    CDialog::OnInitDialog();

    // Add "About..." menu item to system menu.

    // IDM_ABOUTBOX must be in the system command range.
    ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
    ASSERT(IDM_ABOUTBOX < 0xF000);

    CMenu* pSysMenu = GetSystemMenu(FALSE);
    if (pSysMenu != NULL)
       {
        BOOL bNameValid;
        CString strAboutMenu;
        bNameValid = strAboutMenu.LoadString(IDS_ABOUTBOX);
        ASSERT(bNameValid);
        if (!strAboutMenu.IsEmpty())
           {
            pSysMenu->AppendMenu(MF_SEPARATOR);
            pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
           }
       }

    // Set the icon for this dialog.  The framework does this automatically
    //  when the application's main window is not a dialog
    SetIcon(m_hIcon, TRUE);                 // Set big icon
    SetIcon(m_hIcon, FALSE);                // Set small icon

    Refresh();

    CFont * f = GetFont();
    c_NodeMap.SetFont(f);

    updateControls();

    return TRUE;  // return TRUE  unless you set the focus to a control
   }

/****************************************************************************
*                        CNumaExplorerDlg::OnSysCommand
* Inputs:
*       UINT nID: Command ID
*       LPARAM lParam: Passed on to superclass
* Result: void
*       
* Effect: 
*       Handes the About... box
****************************************************************************/

void CNumaExplorerDlg::OnSysCommand(UINT nID, LPARAM lParam)
   {
    if ((nID & 0xFFF0) == IDM_ABOUTBOX)
       {
        CAboutDlg dlgAbout;
        dlgAbout.DoModal();
       }
    else
       {
        CDialog::OnSysCommand(nID, lParam);
       }
   }

/****************************************************************************
*                      CNumaExplorerDlg::OnQueryDragIcon
* Result: HCURSOR
*       
* Effect: 
*       The system calls this function to obtain the cursor to display while
*       the user drags the minimized window.         
****************************************************************************/

HCURSOR CNumaExplorerDlg::OnQueryDragIcon()
{
        return static_cast<HCURSOR>(m_hIcon);
}

/****************************************************************************
*                            CNumaExplorerDlg::OnOK
* Result: void
*       
* Effect: 
*       Does nothing.
****************************************************************************/

void CNumaExplorerDlg::OnOK()
{
}

/****************************************************************************
*                          CNumaExplorerDlg::OnCancel
* Result: void
*       
* Effect: 
*       Does nothing
****************************************************************************/

void CNumaExplorerDlg::OnCancel()
   {
   }

/****************************************************************************
*                          CNumaExplorerDlg::OnClose
* Result: void
*       
* Effect: 
*       Closes the app
****************************************************************************/

void CNumaExplorerDlg::OnClose()
{
    CDialog::OnOK();
}

/****************************************************************************
*                           CNumaExplorerDlg::OnSize
* Inputs:
*       UINT nType:
*       int cx:
*       int cy:
* Result: void
*       
* Effect: 
*       Resizes appropriate controls
****************************************************************************/

void CNumaExplorerDlg::OnSize(UINT nType, int cx, int cy)
   {
    CDialog::OnSize(nType, cx, cy);

    if(c_NumaMemoryMap.GetSafeHwnd() != NULL)
       { /* resize NUMA memory map */
        CRect r;
        c_NumaMemoryMap.GetWindowRect(&r);
        ScreenToClient(&r);
        c_NumaMemoryMap.SetWindowPos(NULL, 0, 0, cx, cy - r.top, SWP_NOMOVE | SWP_NOZORDER);
       } /* resize NUMA memory map */

    if(c_GLPI.GetSafeHwnd() != NULL)
       { /* resize GLPI */
        CRect r;
        c_GLPI.GetWindowRect(&r);
        ScreenToClient(&r);
        c_GLPI.SetWindowPos(NULL, 0, 0, cx - r.left, r.Height(), SWP_NOMOVE | SWP_NOZORDER);
       } /* resize GLPI */

    if(c_Log.GetSafeHwnd() != NULL)
       { /* resize Log */
        CRect r;
        c_Log.GetWindowRect(&r);
        ScreenToClient(&r);
        c_Log.SetWindowPos(NULL, 0, 0, cx - r.left, r.Height(), SWP_NOMOVE | SWP_NOZORDER);
       } /* resize Log */

    if(c_NodeInfo.GetSafeHwnd() != NULL)
       { /* resize nodeinfo */
        CRect r;
        c_NodeInfo.GetWindowRect(&r);
        ScreenToClient(&r);
        c_NodeInfo.SetWindowPos(NULL, 0, 0, cx - r.left, r.Height(), SWP_NOMOVE | SWP_NOZORDER);
       } /* resize nodeinfo */

    if(c_NodeMap.GetSafeHwnd() != NULL)
       { /* resize nodemap */
        CRect r;
        c_NodeMap.GetWindowRect(&r);
        ScreenToClient(&r);
        c_NodeMap.SetWindowPos(NULL, 0, 0, cx - r.left, r.Height(), SWP_NOMOVE | SWP_NOZORDER);
       } /* resize nodemap */

    if(c_XML.GetSafeHwnd() != NULL)
       { /* resize XML */
        CRect r;
        c_XML.GetWindowRect(&r);
        ScreenToClient(&r);
        c_XML.SetWindowPos(NULL, 0, 0, cx - r.left, r.Height(), SWP_NOMOVE | SWP_NOZORDER);
       } /* resize XML */

    if(c_SYSTEM_INFO.GetSafeHwnd() != NULL)
       { /* resize SYSTEM_INFO */
        CRect r;
        c_SYSTEM_INFO.GetWindowRect(&r);
        ScreenToClient(&r);
        c_SYSTEM_INFO.SetWindowPos(NULL, 0, 0, cx - r.left, r.Height(), SWP_NOMOVE | SWP_NOZORDER);
       } /* resize SYSTEM_INFO */
   }

/****************************************************************************
*                      CNumaExplorerDlg::updateControls
* Result: void
*       
* Effect: 
*       Updates the controls
****************************************************************************/

void CNumaExplorerDlg::updateControls()
    {
     c_XML.EnableWindow(c_XML.GetCount() != 0);
     c_NodeInfo.EnableWindow(c_NodeInfo.GetCount() != 0);
    } // CNumaExplorerDlg::updateControls

/****************************************************************************
*                     CNumaExplorerDlg::OnBnClickedRefresh
* Result: void
*       
* Effect: 
*       Refreshes
****************************************************************************/

void CNumaExplorerDlg::OnBnClickedRefresh()
   {
    Refresh();
    updateControls();
   }

/****************************************************************************
*                      CNumaExplorerDlg::OnGetMinMaxInfo
* Inputs:
*       MINMAXINFO * lpMMI:
* Result: void
*       
* Effect: 
*       Limits the drag size
****************************************************************************/

void CNumaExplorerDlg::OnGetMinMaxInfo(MINMAXINFO* lpMMI)
   {
    if(c_Frame.GetSafeHwnd() != NULL)
       { /* can limit */
        CRect r;
        c_Frame.GetWindowRect(&r);
        ScreenToClient(&r);
        if(GetMenu() != NULL)
           r.bottom += ::GetSystemMetrics(SM_CYMENU);
        CalcWindowRect(&r);
        lpMMI->ptMinTrackSize.x = r.Width();
        lpMMI->ptMinTrackSize.y = r.Height();
        return;
       } /* can limit */

    CDialog::OnGetMinMaxInfo(lpMMI);
   }

/****************************************************************************
*                  CNumaExplorerDlg::OnHelpAboutNumaExplorer
* Result: void
*       
* Effect: 
*       Pops up the About... box
****************************************************************************/

void CNumaExplorerDlg::OnHelpAboutNumaExplorer()
    {
     CAboutDlg dlg;
     dlg.DoModal();
    } // CNumaExplorerDlg::OnHelpAboutNumaExplorer

/****************************************************************************
*                          CNumaExplorerDlg::OnFileExit
* Result: void
*       
* Effect: 
*       Exits
****************************************************************************/

void CNumaExplorerDlg::OnFileExit()
    {
     CDialog::OnOK();
    } // CNumaExplorerDlg::OnFileExit

/****************************************************************************
*                      CNumaExplorerDlg::OnEditClearLog
* Result: void
*       
* Effect: 
*       Clears the log
****************************************************************************/

void CNumaExplorerDlg::OnEditClearLog()
    {
     c_Log.ResetContent();
    } // CNumaExplorerDlg::OnEditClearLog

/****************************************************************************
*                     CNumaExplorerDlg::OnFileSaveXMLFile
* Result: void
*       
* Effect: 
*       Writes the stateout as an XML File
* Syntax:
*       <xml header>
*       <NUMA:Data>
*          <NUMA:GetNumaHighestNodeNumber Value="1"/>
*          <NUMA:SYSTEM_INFO dwNumberOfProcessors="4"/>
*
*          <NUMA:GetNumaProcessorNode>
*             <NUMA:ProcessorNode Processor="0" Node="0"/>
*             <NUMA:ProcessorNode Processor="1" Node="0"/>
*          </NUMA:GetNumaProcessorNode>
*
*          <NUMA:GetNumaNodeProcessorMask>
*             <NUMA:ProcessorMask Node="0" ProcessorMask="0x000000003"/>
*             <NUMA:ProcessorMask Node="1" ProcessorMask="0x00000000C"/>
*          </NUMA:GetNumaNodeProcessorMask>
*
*          <NUMA:GetNumaAvailableMemory>
*             <NUMA:NumaAvailableMenory  Node="0" KBytes="0"/>
*             <NUMA:NumaAvailableMenory  Node="1" KBytes="30348"/>
*          </NUMA:GetNumaAvailableMemory>
*
*          <NUMA:GetLogicalProcessorInformation>
*             <NUMA:RelationProcessorCore ProcessorMask="0x00000001" Flags="0x00"/>
*             <NUMA:RelationCache ProcessorMask="0x0000001" Level="1" Associativity="2" LineSize="64" Type="0" TypeName="CacheData"/>
*             <NUMA:RelationProcessorPackage ProcessorMask="0x0000003"/>
*             <NUMA:RelationNumaNode ProcessorMask="0x0000003" Node="0"/>
*          </NUMA:GetLogicalProcessorInformation>
*
*          <NUMA:VirtualAllocExNuma>
*             <NUMA:Allocation CPU="0" Low="0x12345678" High="0x23456789" Node="0"/>
*             <NUMA:Allocation CPU="1" Low="0x12345678" High="0x23456789" Node="0"/>
*          </NUMA:VirtualAllocExNuma>
*
*          <NUMA:QueryWorkingSetEx>
*             <NUMA:PSAPI_WORKING_SET_EX_INFORMATION VirtualAddress="0x12345678" Flags="0x00000000"
*                          Valid="false"/>
*             <NUMA:PSAPI_WORKING_SET_EX_INFORMATION VirtualAddress="0x12345678" Flags="0x00000000"
*                          Valid="true" ShareCount="1" Win32Proection="0x0000" Shared="true" Node="0"
*                          Locked="false" LargePage="false"/>
*          </NUMA:QueryWorkingSetEx>
*       </NUMA:Data>
****************************************************************************/

void CNumaExplorerDlg::OnFileSaveXMLFile()
    {
     CString filter;
     filter.LoadString(IDS_NUMA_FILTER);
     
     CFileDialog dlg(FALSE, _T(".numa"), NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
                     filter);
     if(dlg.DoModal() != IDOK)
        return;

     CStdioFile f;
     CString filename = dlg.GetPathName();
     if(!f.Open(filename, CFile::modeCreate | CFile::modeWrite))
        { /* open failed */
         DWORD err = ::GetLastError();
         c_Log.AddString(new TraceFormatError(TraceEvent::None, IDS_LOG_FILE_OPEN_FAILED, filename));
         c_Log.AddString(new TraceFormatMessage(TraceEvent::None, err));
         CString fmt(MAKEINTRESOURCE(IDS_FILE_OPEN_FAILED));
         CString s;
         s.Format(fmt, filename, ErrorString(err));
         AfxMessageBox(s, MB_ICONERROR | MB_OK);
         return;
        } /* open failed */

     CString xmldata = XMLToString(Data);
     f.WriteString(xmldata);
     f.Close();
    } // CNumaExplorerDlg::OnFileSaveXMLFile

/****************************************************************************
*                     CNumaExplorerDlg::OnFileLoadXMLFile
* Result: void
*       
* Effect: 
*       Loads an XML file
****************************************************************************/

void CNumaExplorerDlg::OnFileLoadXMLFile()
    {
     AfxMessageBox(_T("XML input not yet implemented"), MB_ICONERROR | MB_OK);
    } // CNumaExplorerDlg::OnFileLoadXMLFile


/****************************************************************************
*                         CNumaExplorerDlg::OnDestroy
* Result: void
*       
* Effect: 
*       Cleans up storage
****************************************************************************/

void CNumaExplorerDlg::OnDestroy()
   {
    CDialog::OnDestroy();

    delete Data;
    Data = NULL;
   }

/****************************************************************************
*                        CNumaExplorerDlg::OnQueryNode
* Inputs:
*       WPARAM: ignored
*       LPARAM: ignored
* Result: LRESULT
*       (LRESULT)(XMLNode*): A valid XML node or NULL if none available
****************************************************************************/

LRESULT CNumaExplorerDlg::OnQueryNode(WPARAM, LPARAM)
    {
     return (LRESULT)Data;
    } // CNumaExplorerDlg::OnQueryNode


/****************************************************************************
*                      CNumaExplorerDlg::OnInitMenuPopup
* Inputs:
*       CMenu * pPopupMenu: ignored except to pass to superclass
*       UINT nIndex: ignored except to pass to superclass
*       BOOL bSysMenu: ignored except to pass to superclass
* Result: void
*       
* Effect: 
*       Enables/disables menu items
****************************************************************************/

void CNumaExplorerDlg::OnInitMenuPopup(CMenu* pPopupMenu, UINT nIndex, BOOL bSysMenu)
   {
    CDialog::OnInitMenuPopup(pPopupMenu, nIndex, bSysMenu);

    GetMenu()->EnableMenuItem(ID_EDIT_CLEARLOG, MF_BYCOMMAND | (c_Log.GetCount() > 0 ? MF_ENABLED : MF_GRAYED));
   }
