#include "stdafx.h"
#include "resource.h"
#define XMLEXCEPTION_0B5F600D_7BF1_471e_B1AD_B3BE2A12445B
#include "xmlexceptions.h"
#include "ToString.h"

/****************************************************************************
*                        CXMLException::CXMLException
* Inputs:
*       int ln: Line # of input file
* Effect: 
*       Constructor
****************************************************************************/

CXMLException::CXMLException(int ln)
    {
     line = ln;
    } // CXMLException::CXMLException

/****************************************************************************
*                        CXMLException::~CXMLException
* Effect: 
*       Destructor
****************************************************************************/

CXMLException::~CXMLException()
    {
    } // CXMLException::~CXMLException

/****************************************************************************
*          CXMLDuplicateValueException::CXMLDuplicateValueException
* Inputs:
*       const CString & Old: Old value
*       int oldln: Input line on which old value was defined
*       const CString & nm: name of value
*       const CString & val: New value being set
*       int ln (= 0): Input line on which new value was defined
* Effect: 
*       Constructor
****************************************************************************/

CXMLDuplicateValueException::CXMLDuplicateValueException(const CString & Old, int oldln, const CString & nm, const CString & val, int ln /* = 0 */)
      : CXMLException(ln)
    {
     OldValue = Old;
     OldLine = oldln;
     Name = nm;
     NewValue = val;
    } // CXMLDuplicateValueException::CXMLDuplicateValueException

/****************************************************************************
*          CXMLDuplicateValueException::~CXMLDuplicateValueException
* Effect: 
*       Destructor
****************************************************************************/

CXMLDuplicateValueException::~CXMLDuplicateValueException()
    {
    } // CXMLDuplicateValueException::~CXMLDuplicateValueException

/****************************************************************************
*                  CXMLSyntaxException::~CXMLSyntaxException
* Effect: 
*       Destructor
****************************************************************************/

CXMLSyntaxException::~CXMLSyntaxException()
    {
    } // CXMLSyntaxException::~CXMLSyntaxException

/****************************************************************************
*                          CXMLException::GetString
* Result: CString
*       
* Effect: 
*       Returns a printable string
****************************************************************************/

CString CXMLException::GetString()
    {
     return ToString(IDS_GENERIC_XMLEXCEPTION, line);
    } // CXMLException::GetString

/****************************************************************************
*                          CXMLExpectedNameException
* Result: CString
*       
****************************************************************************/

CString CXMLExpectedNameException::GetString()
    {
     return ToString(IDS_EXPECTED_NAME, line, Found);
    } // CXMLExpectedNameException

/****************************************************************************
*                       CXMLSyntaxException::GetString
* Result: CString
*       
****************************************************************************/

CString CXMLSyntaxException::GetString()
    {
     return ToString(IDS_SYNTAX_ERROR, line);
    } // CXMLSyntaxException::GetString

/****************************************************************************
*                    CXMLExpectedTokenException::GetString
* Result: CString
*       
****************************************************************************/

CString CXMLExpectedTokenException::GetString()
    {
     return ToString(IDS_EXPECTED_TOKEN, line, Expected, Found);
    } // CXMLExpectedTokenException::GetString
    

/****************************************************************************
*                    CXMLExpectedValueException::GetString
* Result: CString
*       
****************************************************************************/

CString CXMLExpectedValueException::GetString()
    {
     return ToString(IDS_EXPECTED_VALUE, line, Found);
    } // CXMLExpectedValueException::GetString

/****************************************************************************
*                   CXMLDuplicateValueException::GetString
* Result: CString
*       
****************************************************************************/

CString CXMLDuplicateValueException::GetString()
    {
     // (%d): Duplicate value %s="%s" (previously defined as "%s" on line %d)
     return ToString(IDS_DUPLICATE_VALUE, line, Name, NewValue, OldValue, OldLine);
    } // CXMLDuplicateValueException

/****************************************************************************
*                    CXMLUnmatchedCloseException::GetString
* Result: CString
*       
****************************************************************************/

CString CXMLUnmatchedCloseException::GetString()
    {
     // (%d): Unmatched close </%s%s%s> does not match (%d) <%s%s%s>
     CString CloseColon = CloseTag.IsEmpty() ? _T("") : _T(":");
     CString OpenColon  = OpenTag.IsEmpty()  ? _T("") : _T(":");
     return ToString(IDS_UNMATCHED_CLOSE, CloseLine, CloseTag, CloseColon, CloseName, OpenLine, OpenTag, OpenColon, OpenName);
    } // CXMLUnmatchedCloseException::GetString

/****************************************************************************
*                 CXMLUnterminatedCommentException::GetString
* Result: CString
*       
****************************************************************************/

CString CXMLUnterminatedCommentException::GetString()
    {
     return ToString(IDS_UNTERMINATED_COMMENT, line);
    } // CXMLUnterminatedCommentException::GetString

/****************************************************************************
*                  CXMLSyntaxException::CXMLSyntaxException
* Inputs:
*       int ln: Line on which error occurred
*       const CString & source: Source input string
*       int offset: Offset into input string where error was detected
* Effect: 
*       Constructor
****************************************************************************/

CXMLSyntaxException::CXMLSyntaxException(int ln, const CString & source, int offset)
: CXMLException(ln)
    {
     // Extract source text
     int start = offset;
     int begin = 0;
     int pastlines = 3; // TODO: decide optimal value for this...
     while(pastlines > 0)
        { /* scan backwards */
         for(int i = start; i >= 0; i--)
            { /* scan backward to start of line */
             if(source[i] == _T('\n'))
                { /* found prev newline */
                 begin = i + 1;
                 start = i - 1;
                 break;
                } /* found prev newline */
            } /* scan backward to start of line */
         pastlines--;
        } /* scan backwards */
     int end = offset;
     for( ; end < source.GetLength(); end++)
        if(source[end] == _T('\n'))
           break;

     //  text \n ... \n    lexeme      \n
     //  ^                       ^      ^
     //  |                       |      |
     //  start              offset      end
     
     SourceText = source.Mid(begin, (offset - start) - 2);
     SourceText += _T("^");
     SourceText += source.Mid(offset, end - offset);
     
    } // CXMLSyntaxException::CXMLSyntaxException
