#pragma once

/****************************************************************************
*                                  class XML
* The fundamental parser class
****************************************************************************/

class XML {
    public:
       XMLNode * Build(const CString & text);
       CString UnBuild(const XMLNode * node, BOOL CRLF);
       static void SetNotification(CWnd * w, UINT m) { wnd = w; msg = m; }
    protected:
       typedef enum {Error,
                     OpenQuest,        // <?
                     CloseQuest,       // ?>
                     Open,             // <
                     Close,            // >
                     OpenComment,      // <!--
                     CloseComment,     // -->
                     SlashClose,       // />
                     Colon,            // :
                     Equal,            // =
                     Slash,            // /
                     Name,             // [A-Za-z0-9_]+
                     Value,            // "value"
                     Integer,          // "[+-]?[0x|0X]?[0-9a-fA-F]+"
                     Other,            // any character not otherwise recognized
                     Empty             //
       } TokenType;
    protected:
       void BuildNode(XMLNode * parent, const CString & source, int & offset, int depth);
       BOOL ParseNode(XMLNode * parent, const CString & source, int & offset, int depth);
       void ParseXMLHeader(XMLNode * parent, const CString & source, int & offset, int depth);       
       void ParseValues(XMLNode * current, const CString & source, int & offset, TokenType & token, CString & lexeme, int depth);
       void ParseComment(XMLNode * parent, const CString & source, int & offset, int depth);
       void ParseClose(XMLNode * parent, const CString & source, int & offset, int depth);
    protected:
       static void lexer(CString & result, TokenType & token, BOOL incomment, int & offset, const CString & source, int & line);
       static BOOL IsInteger(const CString & s, DWORD & value);     // REQ #012
       static BOOL IsInteger(const CString & s, int & value)  { return IsInteger(s, (DWORD&)value); }// REQ #012
       static BOOL IsInteger(const CString & s, UINT & value) { return IsInteger(s, (DWORD&)value); }// REQ #012
       static BOOL IsInteger(const CString & s, long & value) { return IsInteger(s, (DWORD&)value); }// REQ #012
       static CWnd * wnd;
       static UINT msg;
#ifdef _DEBUG
       static CString cvToken(TokenType token);
#endif
    protected:
       int line;    
    }; // class XML
